import { Request } from '../models/request';
import { Record, RefinementResults, Results } from '../models/response';
import { Query } from './query';
export interface RawRecord extends Record {
    _id: string;
    _u: string;
    _t: string;
    _snippet?: string;
}
export interface BridgeCallback {
    (err?: Error, res?: Results): void;
}
export declare type BridgeQuery = string | Query | Request;
export declare const DEFAULT_CONFIG: BridgeConfig;
export declare abstract class AbstractBridge {
    config: BridgeConfig;
    headers: any;
    baseUrl: string;
    errorHandler: (error: Error) => void;
    protected bridgeUrl: string;
    protected refinementsUrl: string;
    constructor(config: BridgeConfig);
    search(query: BridgeQuery, callback?: BridgeCallback): Promise<Results>;
    refinements(query: BridgeQuery, navigationName: string, callback?: BridgeCallback): Promise<RefinementResults>;
    protected abstract augmentRequest(request: any): any;
    private handleResponse<T>(response, callback);
    private extractRequest(query);
    private generateError(error, callback);
    private fireRequest(url, body, queryParams?);
    private convertRecordFields(record);
}
export declare class CloudBridge extends AbstractBridge {
    private clientKey;
    constructor(clientKey: string, customerId: string, config?: BridgeConfig);
    protected augmentRequest(request: any): any;
}
export declare class BrowserBridge extends AbstractBridge {
    constructor(customerId: string, https?: boolean, config?: BridgeConfig);
    protected augmentRequest(request: any): any;
}
export interface BridgeConfig {
    timeout?: number;
}
